#!/usr/bin/python
# -*- coding: utf-8 -*-
import pytest


@pytest.fixture(scope="module")
def new_incoming():
    return {
        "text": "**good** incoming",
        "markdown": True,
        "channel": "testing",
        "attachments": [{
            "title": "title_1",
            "text": "attachment_text",
            "color": "#ffa500",
            "images": [{
                "url": "http://img3.douban.com/icon/ul15067564-30.jpg"
            }]
        }]
    }



#!/usr/bin/python
# -*- coding: utf-8 -*-
import requests


def validate(data):
    """Validates incoming data

    Args:
        data(dict): the incoming data

    Returns:
        True if the data is valid

    Raises:
        ValueError: the data is not valid
    """
    if not isinstance(data, dict):
        raise ValueError("data should be dict")
    if "text" not in data or not isinstance(data["text"],
                                            str) or len(data["text"]) < 1:
        raise ValueError("text field is required and should not be empty")
    if "markdown" in data and not isinstance(data["markdown"], bool):
        raise ValueError("markdown field should be bool")

    if "attachments" in data:
        if not isinstance(data["attachments"], list):
            raise ValueError("attachments field should be list")
        for attachment in data["attachments"]:
            if "text" not in attachment and "title" not in attachment:
                raise ValueError("text or title is required in attachment")

    return True


def send(url, data):
    """Sends an incoming message

    Args:
        url(str): the incoming hook url
        data(dict): the sending data

    Returns:
        requests.Response
    """
    validate(data)
    return requests.post(url, json=data)



#!/usr/bin/python
# -*- coding: utf-8 -*-
from requests import Request, Session
from .rtm_client_service import RTMChannel, RTMCurrentTeam, RTMUser


class RTMResponse(object):
    """Response of Real Time Message

    Attributes:
        resp(requests.Response): HTTP response object
        data(dict): the received data
    """

    def __init__(self, http_resp):
        self.resp = http_resp
        try:
            self.data = self.resp.json()
        except:
            self.data = {}

    def is_ok(self):
        """
        Returns:
            bool: True if the response is valid
        """
        return self.resp.status_code == 200 and self.data.get("code") == 0

    def is_fail(self):
        return not self.is_ok()


class RTMClient(object):
    """Real Time Message client

    Attributes:
        current_team(RTMCurrentTeam): service of current team
        user(RTMUser): service of current user
        channel(RTMChannel): service of current channel
    """

    def __init__(self, token, api_base="https://rtm.bearychat.com"):
        """
        Args:
            token(str): rtm token
            api_base(str): api url base
        """
        self._token = token
        self._api_base = api_base
        self.current_team = RTMCurrentTeam(self)
        self.user = RTMUser(self)
        self.channel = RTMChannel(self)

    def start(self):
        """Gets the rtm ws_host and user information

        Returns:
            None if request failed,
            else a dict containing "user"(User) and "ws_host"
        """
        resp = self.post("start")
        if resp.is_fail() or "result" not in resp.data:
            return None
        return {
            "user": resp.data["result"]["user"],
            "ws_host": resp.data["result"]["ws_host"]
        }

    def do(self,
           resource,
           method,
           params=None,
           data=None,
           json=None,
           headers=None):
        """Does the request job

        Args:
            resource(str): resource uri(relative path)
            method(str): HTTP method
            params(dict): uri queries
            data(dict): HTTP body(form)
            json(dict): HTTP body(json)
            headers(dict): HTTP headers

        Returns:
            RTMResponse
        """
        uri = "{0}/{1}".format(self._api_base, resource)
        if not params:
            params = {}
        params.update({"token": self._token})

        req = Request(
            method=method,
            url=uri,
            params=params,
            headers=headers,
            data=data,
            json=json)
        s = Session()
        prepped = s.prepare_request(req)
        resp = s.send(prepped)

        return RTMResponse(resp)

    def get(self, resource, params=None, headers=None):
        """Sends a GET request

        Returns:
            RTMResponse
        """
        return self.do(resource, "GET", params=params, headers=headers)

    def post(self, resource, data=None, json=None):
        """Sends a POST request

        Returns:
            RTMResponse
        """
        return self.do(resource, "POST", data=data, json=json)



#!/usr/bin/python
# -*- coding: utf-8 -*-


class UserType(object):
    Normal = "normal"
    Assistant = "assistant"
    Hubot = "hubot"


class UserRole(object):
    Owner = "owner"
    Admin = "admin"
    Normal = "normal"
    Visitor = "visitor"


def is_user_online(user):
    # type: (Dict) -> bool
    return user["conn"] == "connected"


def is_user_normal(user):
    # type: (Dict) -> bool
    return (user["type"] == UserType.Normal and
            user["role"] != UserRole.Visitor)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import json


class RTMMessageType(object):
    """types of Message of Real Time Message
    """
    Unknown = "unknown"
    Ping = "ping"
    Pong = "pong"
    Reply = "reply"
    Ok = "ok"
    P2PMessage = "message"
    P2PTyping = "typing"
    ChannelMessage = "channel_message"
    ChannelTyping = "channel_typing"
    UpdateUserConnection = "update_user_connection"


class RTMMessage(object):
    """Message of Real Time Message
    """
    def __init__(self, data):
        """
        Args:
            data(dict): message content

        Raises:
            TypeError: if data is not dict
        """
        if not isinstance(data, dict):
            raise TypeError
        self._data = data

    def __setitem__(self, name, value):
        self._data[name] = value

    def __getitem__(self, name):
        return self._data[name]

    def __contains__(self, name):
        return name in self._data

    def reply(self, text):
        """Replys a text message

        Args:
            text(str): message content

        Returns:
            RTMMessage
        """
        data = {"text": text, "vchannel_id": self["vchannel_id"]}
        if self.is_p2p():
            data["type"] = RTMMessageType.P2PMessage
            data["to_uid"] = self["uid"]
        else:
            data["type"] = RTMMessageType.ChannelMessage
            data["channel_id"] = self["channel_id"]
        return RTMMessage(data)

    def refer(self, text):
        """Refers current message and replys a new message

        Args:
            text(str): message content

        Returns:
            RTMMessage
        """
        data = self.reply(text)
        data["refer_key"] = self["key"]
        return data

    def is_p2p(self):
        """
        Returns:
            True if current message is p2p message
        """
        t = self["type"]
        return (t == RTMMessageType.P2PMessage or
                t == RTMMessageType.P2PTyping)

    def is_chat_message(self):
        """
        Returns:
            True if current message is chatting message
        """
        t = self["type"]
        return (t == RTMMessageType.P2PMessage or
                t == RTMMessageType.ChannelMessage)

    def is_from(self, user):
        """Checks if current message is sent by user

        Args:
            user(User)

        Returns:
            True if current message is sent by the user
        """
        return self["uid"] == user["id"]

    def to_json(self):
        """Transfers current message to json

        Returns:
            json
        """
        return json.dumps(self._data)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import pytest
from bearychat.model import UserType, UserRole, is_user_online, is_user_normal


def test_is_user_online():
    assert is_user_online({"conn": False}) is False
    assert is_user_online({"conn": 42}) is False
    assert is_user_online({"conn": "disconnected"}) is False
    assert is_user_online({"conn": "connected"}) is True

    with pytest.raises(KeyError):
        is_user_online({})


def test_is_user_normal():
    assert is_user_normal({"type": UserType.Assistant,
                           "role": UserRole.Normal}) is False
    assert is_user_normal({"type": 42,
                           "role": UserRole.Normal}) is False
    assert is_user_normal({"type": UserType.Normal,
                           "role": UserRole.Visitor}) is False
    assert is_user_normal({"type": UserType.Normal,
                           "role": UserRole.Normal}) is True
    with pytest.raises(KeyError):
        is_user_normal({})



#!/usr/bin/python
# -*- coding: utf-8 -*-
import pytest
from bearychat.incoming import validate


def test_validate_success(new_incoming):
    assert validate(new_incoming) is True


def test_validate_data_type(new_incoming):
    assert validate(new_incoming) is True
    ts = [list, str, int, float, bool]
    for t in ts:
        incoming = t()
        with pytest.raises(ValueError, message="data should be dict"):
            validate(incoming)


def test_validate_markdown_field(new_incoming):
    ts = [list, str, int, float, dict]
    for t in ts:
        new_incoming["markdown"] = t()
        with pytest.raises(
                ValueError, message="markdown field should be bool"):
            validate(new_incoming)

    new_incoming["markdown"] = True
    assert validate(new_incoming) is True


def test_validate_text_field(new_incoming):
    ts = [list, int, float, dict, bool, str]
    for t in ts:
        new_incoming["text"] = t()
        with pytest.raises(
                ValueError,
                message="text field is required and should not be empty"):
            validate(new_incoming)
    new_incoming["text"] = "testing"
    assert validate(new_incoming) is True


def test_validate_attachments(new_incoming):
    ts = [int, float, dict, bool, str]
    for t in ts:
        new_incoming["attachments"] = t()
        with pytest.raises(
                ValueError, message="attachments field should be list"):
            validate(new_incoming)
    new_incoming["attachments"] = [{
        "color": "#ffa500",
        "images": [{
            "url": "http://img3.douban.com/icon/ul15067564-30.jpg"
        }]
    }]

    with pytest.raises(
            ValueError, message="text or title is required in attachment"):
        validate(new_incoming)

    new_incoming["attachments"] = [{
        "title": "testing",
        "color": "#ffa500",
        "images": [{
            "url": "http://img3.douban.com/icon/ul15067564-30.jpg"
        }]
    }]
    assert validate(new_incoming) is True

    new_incoming["attachments"] = [{
        "text": "testing",
        "color": "#ffa500",
        "images": [{
            "url": "http://img3.douban.com/icon/ul15067564-30.jpg"
        }]
    }]
    assert validate(new_incoming) is True



from setuptools import setup, find_packages
from os.path import dirname, realpath, join


CURRENT_DIR = dirname(realpath(__file__))

with open(join(CURRENT_DIR, "README.rst")) as long_description_file:
    long_description = long_description_file.read()

setup(
    name="bearychat",
    version="0.2.1",
    author="bearyinnovative",
    url="https://github.com/bearyinnovative/bearychat.py",
    description="SDK for BearyChat",
    long_description=long_description,
    packages=find_packages(exclude=["docs"]),
    platforms="any",
    install_requires=[
        "requests>=2.8.0",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Software Development",
    ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-


class RTMServiceError(RuntimeError):
    def __init__(self, message, err_data):
        super(RTMServiceError, self).__init__(message)
        self.err_data = err_data


class RTMService(object):
    """service of Real Time Message
    """

    def __init__(self, rtm_client):
        self._rtm_client = rtm_client


class RTMCurrentTeam(RTMService):
    """
    """

    def info(self):
        """Gets current team infomation

        Returns:
            Team if success

        Throws:
            RTMServiceError when request failed
        """
        resp = self._rtm_client.get("v1/current_team.info")
        if resp.is_fail():
            raise RTMServiceError("Failed to get current team infomation",
                                  resp)
        return resp.data["result"]

    def members(self):
        """Gets members of current team

        Returns:
            list of User

        Throws:
            RTMServiceError when request failed
        """
        resp = self._rtm_client.get("v1/current_team.members?all=true")
        if resp.is_fail():
            raise RTMServiceError("Failed to get members of current team",
                                  resp)
        members = []
        for member in resp.data["result"]:
            members.append(member)
        return members

    def channels(self):
        """Gets channels of current team

        Returns:
            list of Channel

        Throws:
            RTMServiceError when request failed
        """
        resp = self._rtm_client.get("v1/current_team.channels")
        if resp.is_fail():
            raise RTMServiceError("Failed to get channels of current team",
                                  resp)
        channels = []
        for channel in resp.data["result"]:
            channels.append(channel)
        return channels


class RTMUser(RTMService):
    def info(self, user_id):
        """Gets user information by user id

        Args:
            user_id(int): the id of user

        Returns:
            User

        Throws:
            RTMServiceError when request failed
        """
        resp = self._rtm_client.get("v1/user.info?user_id={0}".format(user_id))
        if resp.is_fail():
            raise RTMServiceError("Failed to get user information", resp)
        return resp.data["result"]


class RTMChannel(RTMService):
    def info(self, channel_id):
        """Gets channel information by channel id

        Args:
            channel_id(int): the id of channel

        Returns:
            Channel

        Throws:
            RTMServiceError when request failed
        """
        resp = self._rtm_client.get("v1/channel.info?channel_id={0}".format(
            channel_id))
        if resp.is_fail():
            raise RTMServiceError("Failed to get channel information", resp)
        return resp.data["result"]
